
from .depends import create_ratelimit_depends
from .error_handler import LimiterErrorHandler
from .limiter_engine import LimiterEngine
from .decorator import create_ratelimit_decorator


class Limiter:

    def __init__(
            self,
            engine: LimiterEngine,
            error_handler: LimiterErrorHandler
    ):
        self.engine = engine
        self.error_handler = error_handler

    @property
    def decorator(self):
        return create_ratelimit_decorator(self.engine, self.error_handler)

    @property
    def depends(self):
        return create_ratelimit_depends(self.engine, self.error_handler)



from typing import List


class Limit:

    def __init__(
            self,
            interval: int,
            max_requests: int
    ):
        self.interval = interval
        self.max_requests = max_requests

    def __repr__(self):
        return f'Limit(interval={self.interval},max_requests={self.max_requests})'


class LimitConfig:

    def __init__(
            self,
            key: str,
            limits: List[Limit],
    ):
        self.key = key
        self.limits = limits

    def __repr__(self):
        return f'LimitConfig(key={self.key},limits={self.limits})'



class LimiterResult:

    def __init__(
            self,
            limit_applied: bool,
            description: str,
            limit_meta: list
    ):
        self.limit_applied = limit_applied
        self.description = description
        self.limit_meta = limit_meta

    def __repr__(self):
        return f'LimiterResult(' \
               f'limit_applied={self.limit_applied},' \
               f'description={self.description},' \
               f'limit_meta={self.limit_meta}' \
               f')'
